/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 *****************************************************************************/
/**
 * @file
 * Header file for CApexException class which defines the methods used for
 * exceptions.
 */


///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////
#pragma once


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include <string>


///////////////////////////////////////////////////////////////////////////////
// enums
///////////////////////////////////////////////////////////////////////////////

/**
 * Severity of the exception
 */
enum EnumSeverity
{
	ERROR_INFORMATION = 1,
	ERROR_WARNING     = 2,
	ERROR_FATAL       = 3,
	ERROR_SYSTEM      = 4
};


///////////////////////////////////////////////////////////////////////////////
// class definition
///////////////////////////////////////////////////////////////////////////////

/**
 * This is the main exception class.
 */
class CApexException
{
public:

	CApexException();
	CApexException( EnumSeverity enumSeverity, const std::wstring& strDescription, const std::string& strFile, long nLineNumber );
	CApexException( EnumSeverity enumSeverity, const std::string& strDescription, const std::string& strFile, long nLineNumber );
	virtual ~CApexException();
	CApexException(const CApexException& ex);
	std::wstring GetDescription(bool bAddFileName = true) const;

#ifdef _MSC_VER //this function only available on Windows
	static std::wstring GetSystemError();
#endif

	CApexException& operator=( const CApexException& rhs );
	void SetFile(const std::string& strFile);
	void SetFile(const std::wstring& strFile);
	void SetDescription(const std::string& strFile);
	void SetDescription(const std::wstring& strFile);
	std::wstring GetFileName() const;
	EnumSeverity GetSeverity() const;
	long GetLineNumber() const;
	const std::wstring& GetFilePath() const;

private:
	/// A wide string that holds the description of the exception.
	std::wstring  m_strDescription;

	/// A wide string that holds the filename where the error occurred.
	std::wstring  m_strFile;

	/// A long that holds the line number within the file where the error occurred.
	long          m_nLineNumber;

	/// An Enum that holds the severity of the exception.
	EnumSeverity  m_enumSeverity;
};
